from flask import redirect, url_for, Blueprint, current_app
from sqlalchemy import MetaData, create_engine

base = Blueprint("base", __name__)

@base.route('/favicon.ico')
def favicon():
    return redirect(url_for('static', filename='img/favicon.png'), 308)

@base.route('/k8srad/liveness', methods=['GET'])
def liveness():
    return '', 200

@base.route('/k8srad/readiness', methods=['GET'])
def readiness():
    meta = MetaData(create_engine(current_app.config['SQLALCHEMY_DATABASE_URI']))
    meta.reflect(views=True)
    if 'ab_user' in meta.tables:
        return '', 200
    return '', 503


import datetime
from sqlalchemy import Boolean
from sqlalchemy.orm import registry
from flask_appbuilder import Model
from sqlalchemy import Column, Integer, String


mapper_registry = registry()

class Version(Model):
    __tablename__ = 'k8srad_version'
    id = Column(Integer, primary_key=True)
    name = Column(String(150), nullable=False, unique=True)
    mayor = Column(Integer, nullable=False, unique=False)
    minor = Column(Integer, nullable=False, unique=False)
    patch = Column(Integer, nullable=False, unique=False)
    updated_at = Column(String(150), default=datetime.datetime.now().isoformat(), nullable=False)
    is_current = Column(Boolean, nullable=False, unique=False)



import os, logging
import sys
from datetime import timedelta
from waitress import serve
from dotenv import dotenv_values
from flask import Flask
from flask_appbuilder import SQLA, AppBuilder
from flask_session import Session
from werkzeug.middleware.proxy_fix import ProxyFix
from k8srad.views import VersionView
from k8srad.server import ServerAroParser
from k8srad.sqls.sql import Sqls
from k8srad.blueprint import base
from flask_appbuilder.security.manager import (
    AUTH_DB,
    AUTH_LDAP,
)

basedir = os.path.abspath(os.path.dirname(__file__))
# init Flask
app = Flask(__name__)
app.wsgi_app = ProxyFix(app.wsgi_app, x_proto=1)
app.register_blueprint(base)

app.config.from_pyfile(basedir + "/config.py")
log_level = logging.INFO
if os.getenv("K8SRAD_CONF_FILE") is not None:
    dotenv_path = os.getenv("K8SRAD_CONF_FILE")
    if not os.path.isfile(dotenv_path):
        print("ERROR: Cannot find .env File in {path}. Cannot start.".format(path=dotenv_path))
        sys.exit(1)
    app.config.update(dotenv_values(dotenv_path))
    log_level = logging.DEBUG if app.config['K8SRAD_LOG_LEVEL'].upper() == "DEBUG" else log_level
    log_level = logging.WARNING if app.config['K8SRAD_LOG_LEVEL'].upper() == "WARNING" else log_level
    log_level = logging.FATAL if app.config['K8SRAD_LOG_LEVEL'].upper() == "FATAL" else log_level
    log_level = logging.INFO if app.config['K8SRAD_LOG_LEVEL'].upper() == "INFO" else log_level
    log_level = logging.ERROR if app.config['K8SRAD_LOG_LEVEL'].upper() == "ERROR" else log_level

logging.getLogger().setLevel(log_level)
log_format = "%(asctime)s:%(levelname)s:%(name)s:%(message)s"

if 'K8SRAD_LOG_FILE' in app.config:
    try:
        with open(app.config['K8SRAD_LOG_FILE'], 'w') as f:
            f.write('K8SRAD Logfile')
        logging.basicConfig(filename=app.config['K8SRAD_LOG_FILE'], format=log_format)
    except FileNotFoundError as e:
        print("Cannot create log file. Stop processing. Error: {error}".format(error=e))
        sys.exit(1)
else:
    logging.basicConfig(format=log_format)

app.logger.info("The APP is running in {LOGLEVEL} mode.".format(LOGLEVEL=logging.getLevelName(log_level)))

if "K8SRAD_APP_NAME" in app.config:
    app.config["APP_NAME"] = app.config["K8SRAD_APP_NAME"]
app.logger.info("Starting K8SRAD with name {APP_NAME}.".format(APP_NAME=app.config["APP_NAME"]))

if "K8SRAD_APP_ICON" in app.config:
    app.config["APP_ICON"] = app.config["K8SRAD_APP_ICON"]
app.logger.info("Adding {APP_ICON} as App icon.".format(APP_ICON=app.config["APP_ICON"]))

app.config["AUTH_TYPE"] = AUTH_DB
if app.config["K8SRAD_AUTH_TYPE"] == "ldap":
    app.config["AUTH_TYPE"] = AUTH_LDAP

app.config["PERMANENT_SESSION_LIFETIME"] = timedelta(minutes=10)
if "K8SRAD_SESSION_LIFETIME" in app.config:
    app.config["PERMANENT_SESSION_LIFETIME"] = timedelta(seconds=int(app.config["K8SRAD_SESSION_LIFETIME"]))

# Init Database for Server Sessions
server_session = Session(app)
server_session.app.session_interface.db.create_all()

db = SQLA(app)
db.create_all()

# Init Appbuilder Authentication and Authorization
appbuilder = AppBuilder(app, db.session)
appbuilder.add_view(VersionView, "Versions", category="Security", icon="fa-list")
admin = appbuilder.sm.find_user(username="admin")

# Setup default Database
if admin == None:
    role_admin = appbuilder.sm.find_role(
        appbuilder.sm.auth_role_admin
    )
    admin = appbuilder.sm.add_user(
        username="admin",
        first_name="admin",
        last_name="admin",
        email="info@yotron.de",
        role=role_admin,
        password="admin"
    )
    db.session.add(admin)
    db.session.commit()

# Init and Migrate
sqls = Sqls(app)
sqls.update_1_0_0()


def run():
    argParser = ServerAroParser()
    args = argParser.parser.parse_args()
    serve(app, **argParser.getKw(args))



import os

from flask import current_app
from sqlalchemy import create_engine, text
from sqlalchemy.orm import scoped_session, sessionmaker
from k8srad.models import Version


class Sqls:
    engine = None
    session = None
    folderPath = ""


    def __init__(self, app=None):
        if app is None:
            self.engine = create_engine(current_app.config['SQLALCHEMY_DATABASE_URI'])
        else:
            self.engine = create_engine(app.config['SQLALCHEMY_DATABASE_URI'])
        self.session = scoped_session(sessionmaker(autocommit=False,
                                                   autoflush=False,
                                                   bind=self.engine))
        self.folderPath = os.path.dirname(os.path.abspath(__file__))

    def update_1_0_0(self):
        ma, mi, pa = 1, 0, 0
        if self.session.query(Version).count() == 0:
            self.setNewVersion(ma, mi, pa)


    def executeSQLScript(self, file_path):
        with self.engine.connect() as con:
            with open(file_path) as file:
                query = text(file.read())
                con.execute(query)

    def upgradeNeeded(self, mayor, minor, patch):
        curr_mayor, curr_minor, curr_patch = self.getCurrentVersion()
        if curr_mayor < mayor or curr_minor < minor or curr_patch < patch:
            return True
        return False

    def getCurrentVersion(self):
        current_version = self.session.query(Version).filter_by(is_current=True).first()
        return current_version.mayor, current_version.minor, current_version.patch

    def setNewVersion(self, mayor, minor, patch):
        curr_vers = self.session.query(Version).filter_by(is_current=True).all()
        for curr_ver in curr_vers:
            curr_ver.is_current = False
            self.session.commit()
        newVersion = Version(
            name="{}.{}.{}".format(mayor, minor, patch),
            mayor=mayor,
            minor=minor,
            patch=patch,
            is_current=True
        )
        self.session.add(newVersion)
        self.session.commit()





import os
from datetime import timedelta

# check env files
FLASK_DEBUG = 0
K8SRAD_DB_TYPE = "sqlite"
K8SRAD_PG_DB_HOST = ""
K8SRAD_PG_DB_NAME = "k8srad"
K8SRAD_PG_DB_PORT = 5432
K8SRAD_PG_DB_USER_PW = "k8srad"
K8SRAD_PG_DB_USER_NAME = "k8srad"
K8SRAD_AUTH_TYPE = "database"

SECRET_KEY = "thisIsMyHiddenSecretKey"  # from K8SRAD_SECRET_KEY

basedir = os.path.abspath(os.path.dirname(__file__))

# The SQLAlchemy connection string.


SQLALCHEMY_DATABASE_URI = "sqlite:///" + os.path.join(basedir, "../app.db")
# SQLALCHEMY_DATABASE_URI = 'mysql://myapp@localhost/myapp'
if K8SRAD_DB_TYPE == "postgres":
    SQLALCHEMY_DATABASE_URI = 'postgresql://{username}:{userpw}@{dburl}:{dbport}/{dbname}'.format(
        username=K8SRAD_PG_DB_USER_NAME, userpw=K8SRAD_PG_DB_USER_PW, dburl=K8SRAD_PG_DB_HOST, dbport=K8SRAD_PG_DB_PORT,
        dbname=K8SRAD_PG_DB_NAME)

SESSION_TYPE = 'sqlalchemy'

# Flask-WTF flag for CSRF
CSRF_ENABLED = True

# Customizations
APP_NAME = "K8SRAD by YOTRON"
APP_ICON = "https://www.yotron.de/img/yotron_logo_devops.svg"

# ----------------------------------------------------
# AUTHENTICATION CONFIG
# ----------------------------------------------------
# The authentication type
# AUTH_OID : Is for OpenID
# AUTH_LDAP : Is for LDAP
# AUTH_REMOTE_USER : Is for using REMOTE_USER from web server


# Uncomment to setup Full admin role name
AUTH_ROLE_ADMIN = 'Admin'
AUTH_USER_REGISTRATION_ROLE = "Public"
AUTH_ROLES_SYNC_AT_LOGIN = True

# Uncomment to setup Public role name, no authentication needed
# AUTH_ROLE_PUBLIC = 'Public'

# Will allow user self registration
# AUTH_USER_REGISTRATION = True

# The default user self registration role


# When using LDAP Auth, setup the ldap server
# AUTH_LDAP_SERVER = "ldap://ldapserver.new"

# Uncomment to setup OpenID providers example for OpenID authentication
# OPENID_PROVIDERS = [
#    { 'name': 'Yahoo', 'url': 'https://me.yahoo.com' },
#    { 'name': 'AOL', 'url': 'http://openid.aol.com/<username>' },
#    { 'name': 'Flickr', 'url': 'http://www.flickr.com/<username>' },
#    { 'name': 'MyOpenID', 'url': 'https://www.myopenid.com' }]
# ---------------------------------------------------
# Babel config for translations
# ---------------------------------------------------
# Setup default language
# BABEL_DEFAULT_LOCALE = "en"
# Your application default translation path
# BABEL_DEFAULT_FOLDER = "translations"
# The allowed translation for you app
# LANGUAGES = {
#    "en": {"flag": "gb", "name": "English"},
#    "pt": {"flag": "pt", "name": "Portuguese"},
#    "pt_BR": {"flag": "br", "name": "Pt Brazil"},
#    "es": {"flag": "es", "name": "Spanish"},
#    "de": {"flag": "de", "name": "German"},
#    "zh": {"flag": "cn", "name": "Chinese"},
#    "ru": {"flag": "ru", "name": "Russian"},
#    "pl": {"flag": "pl", "name": "Polish"},
# }
# ---------------------------------------------------
# Image and file configuration
# ---------------------------------------------------
# The file upload folder, when using models with files
UPLOAD_FOLDER = basedir + "/app/static/uploads/"

# The image upload folder, when using models with images
IMG_UPLOAD_FOLDER = basedir + "/app/static/uploads/"

# The image upload url, when using models with images
IMG_UPLOAD_URL = "/static/uploads/"
# Setup image size default is (300, 200, True)
# IMG_SIZE = (300, 200, True)

# Theme configuration
# these are located on static/appbuilder/css/themes
# you can create your own and easily use them placing them on the same dir structure to override
# APP_THEME = "bootstrap-theme.css"  # default bootstrap
# APP_THEME = "cerulean.css"
# APP_THEME = "amelia.css"
# APP_THEME = "cosmo.css"
# APP_THEME = "cyborg.css"
# APP_THEME = "flatly.css"
# APP_THEME = "journal.css"
# APP_THEME = "readable.css"
# APP_THEME = "simplex.css"
# APP_THEME = "slate.css"
# APP_THEME = "spacelab.css"
# APP_THEME = "united.css"
# APP_THEME = "yeti.css"

FAB_ROLES = {}

SESSION_REFRESH_EACH_REQUEST = True



import argparse
import sys

import flask
import flask_login
from flask import redirect, url_for, Blueprint
base = Blueprint("base", __name__)


class ServerAroParser:
    parser = argparse.ArgumentParser

    def error(self, message):
        sys.stderr.write('error: %s\n' % message)
        sys.exit(2)

    def __init__(self):
        self.parser = argparse.ArgumentParser(description='Start parameter for the K8SRAD web server.')
        self.parser.add_argument('-s', '--host', type=str,
                          help='Listener Host IP. Default: 0.0.0.0')
        self.parser.add_argument('-p', '--port', type=int,
                          help='Listener Host Port.  Default: 8080.')
        self.parser.add_argument('-t', '--threads', type=int,
                          help='Threads for parallelization. Default: 4.')


    def getKw(self, args):
        kw = {}
        for arg in vars(args):
            if getattr(args, arg) is not None:
                kw[arg] = getattr(args, arg)
        return kw


import datetime

from flask_appbuilder.models.sqla.interface import SQLAInterface
from flask_appbuilder import ModelView

from k8srad.models import Version


class VersionView(ModelView):
    route_base = '/versions'
    datamodel = SQLAInterface(Version)
    list_columns = [
        'name',
        'mayor',
        'minor',
        'patch',
        'updated_at',
        'is_current'
    ]
    edit_columns = [
        'name'
    ]